@@ -1,9 +1,9 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.
  *
- * Contact: Luc Pionchon <luc.pionchon@nokia.com>
+ * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -30,7 +30,6 @@
 #include "hildon-file-chooser-dialog.h"
 #include "hildon-file-system-private.h"
 #include <hildon-widgets/gtk-infoprint.h>
-#include <osso-log.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkbox.h>
 #include <gtk/gtkentry.h>
@@ -48,12 +47,7 @@
 #include <gdk/gdkx.h>
 #include <stdlib.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#define _(String) dgettext(PACKAGE, String)
-#define MAX_FILENAME_LENGTH_DEFAULT 255 /* If env doesn't define, use this */
+#include "hildon-file-common-private.h"
 #define HILDON_RESPONSE_FOLDER_BUTTON 12345
 /* Common height for filetrees. About 8 lines. Filetree sets default margins, 
     so we need to take them into account. See #9962. */
@@ -134,7 +128,7 @@ enum {
     PROP_MAX_FULL_PATH_LENGTH
 };
 
-void _gtk_file_chooser_install_properties(GObjectClass * klass);
+void hildon_gtk_file_chooser_install_properties(GObjectClass * klass);
 
 /* CopyPaste ends */
 
@@ -363,7 +357,7 @@ hildon_file_chooser_dialog_check_limit(GtkEditable *editable,
   if (entry->text_max_length > 0 && 
       entry->text_length + new_text_length > entry->text_max_length)
       gtk_infoprint(GTK_WINDOW(data), 
-          _("ckdg_ib_maximum_characters_reached"));  
+          HCS("ckdg_ib_maximum_characters_reached"));  
 }
 
 static void insensitive_button(GtkWidget *widget, gpointer data)
@@ -943,11 +937,11 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
   g_return_if_fail(HILDON_IS_FILE_CHOOSER_DIALOG(self));
 
   /* Prevent race condition that can cause multiple
-      subdialogs to be popped up (in a case mainloop
-      is run before subdialog blocks additional clicks. */
+     subdialogs to be popped up (in a case mainloop
+     is run before subdialog blocks additional clicks. */
   if (self->priv->popup_protect)
   {
-    ULOG_INFO("Blocked multiple subdialogs");
+    ULOG_INFO_F("Blocked multiple subdialogs");
     return;
   }
 
@@ -956,18 +950,19 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
   if (self->priv->edited)
   {
     self->priv->edited = FALSE;
-    hildon_file_chooser_dialog_set_current_name(
-      GTK_FILE_CHOOSER(self),
+    hildon_file_chooser_dialog_set_current_name(GTK_FILE_CHOOSER(self),
       gtk_entry_get_text(GTK_ENTRY(self->priv->entry_name)));
   }
 
   backend = _hildon_file_system_model_get_file_system(self->priv->model);
 
-  if (self->priv->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER) {
-    dialog = hildon_file_chooser_dialog_create_sub_dialog
-      (self, GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER);
+  if (self->priv->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
+  {
+    dialog = hildon_file_chooser_dialog_create_sub_dialog(self,
+               GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER);
 
-    while (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
+    while (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+    {
       GtkFilePath *file_path;
       gboolean success;
       GError *error = NULL;
@@ -975,16 +970,15 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
       gchar *uri;
 
       uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
-      ULOG_INFO("About to create folder %s", uri);
+      ULOG_INFO_F("About to create folder %s", uri);
 
       file_path = gtk_file_system_uri_to_path(backend, uri);
-      success =
-        gtk_file_system_create_folder(backend, file_path, &error);
+      success = gtk_file_system_create_folder(backend, file_path, &error);
       gtk_file_path_free(file_path);
 
       if (success) 
       {
-        (void) gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(self), uri);
+        gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(self), uri);
         g_free(uri);
         break;
       }
@@ -992,31 +986,32 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
       g_free(uri);
       g_assert(error != NULL);
       
-      /* GtkFileSystemModelGnomeVFS returns ERROR_FAILED and GtkFileSystemUnix returns
-          ERROR_NONEXISTENT (!!!) if we try to create a folder that already exists. We report
-          other errors normally.
-          GnomeVFS has been changed to return ALREADY EXISTS accordingly. 
-          We now support this only. */
-      if (g_error_matches(error, GTK_FILE_SYSTEM_ERROR,  
-              GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS))
-        message = _("ckdg_ib_folder_already_exists");
-      else
-        message = _("sfil_ni_operation_failed");
+      /* GtkFileSystemModelGnomeVFS returns ERROR_FAILED and
+       * GtkFileSystemUnix returns ERROR_NONEXISTENT (!!!) if we try to
+       * create a folder that already exists. We report other errors
+       * normally. GnomeVFS has been changed to return ALREADY EXISTS
+       * accordingly. We now support this only. */
 
-      ULOG_ERR(error->message);
+      if (g_error_matches(error, GTK_FILE_SYSTEM_ERROR,  
+                          GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS))
+        message = HCS("ckdg_ib_folder_already_exists");
+      else
+        message = _("sfil_ib_create_folder_not_allowed");
+
+      ULOG_ERR_F(error->message);
       gtk_infoprint(GTK_WINDOW(dialog), message);
       g_error_free(error);
 
       hildon_file_chooser_dialog_select_text(self->priv);
     }
-  } else {
-    dialog =
-        hildon_file_chooser_dialog_create_sub_dialog
-            (self, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+  }
+  else
+  {
+    dialog = hildon_file_chooser_dialog_create_sub_dialog(self,
+               GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
-        sync_current_folders(HILDON_FILE_CHOOSER_DIALOG(dialog),
-                             self);
+      sync_current_folders(HILDON_FILE_CHOOSER_DIALOG(dialog), self);
   }
 
   gtk_banner_close(GTK_WINDOW(dialog));
@@ -1060,7 +1055,7 @@ static void response_handler(GtkWidget * widget, gint arg1, gpointer data)
                 priv->edited = FALSE;
                 hildon_file_chooser_dialog_do_autonaming(priv);
                 hildon_file_chooser_dialog_select_text(priv);
-                gtk_infoprint(window, _("ckdg_ib_enter_name"));
+                gtk_infoprint(window, HCS("ckdg_ib_enter_name"));
             } else {
                 GString *illegals = check_illegal_characters(entry_text);
 
@@ -1070,8 +1065,9 @@ static void response_handler(GtkWidget * widget, gint arg1, gpointer data)
                     hildon_file_chooser_dialog_select_text(priv);
                     g_signal_stop_emission_by_name(widget, "response");
 
-                    msg = g_strdup_printf(_("ckdg_ib_illegal_characters_entered"),
-                                          illegals->str);
+                    msg = g_strdup_printf(
+                              HCS("ckdg_ib_illegal_characters_entered"),
+                              illegals->str);
                     gtk_infoprint(window, msg);
                     g_free(msg);
                     g_string_free(illegals, TRUE);
@@ -1091,7 +1087,7 @@ static void response_handler(GtkWidget * widget, gint arg1, gpointer data)
                     {
                         g_signal_stop_emission_by_name(widget, "response");
                         hildon_file_chooser_dialog_select_text(priv);
-                        gtk_infoprintf(window, _("file_ib_name_too_long"), 
+                        gtk_infoprintf(window, HCS("file_ib_name_too_long"), 
                             path_length - self->priv->max_filename_length);
                     }
                     g_free(uri);
@@ -1458,7 +1454,7 @@ hildon_file_chooser_dialog_class_init(HildonFileChooserDialogClass * klass)
                              "from MAX_FILENAME_LENGTH environment variable", 
                              -1, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-    _gtk_file_chooser_install_properties(gobject_class);
+    hildon_gtk_file_chooser_install_properties(gobject_class);
 }
 
 static void hildon_file_chooser_dialog_sort_changed(GtkWidget * item,
@@ -1842,7 +1838,8 @@ gchar *hildon_file_chooser_dialog_get_safe_folder(
   fs = _hildon_file_system_model_get_file_system(self->priv->model);
   g_object_get(self->priv->model, "safe-folder", &path, NULL);
   if (path == NULL)
-    path = _hildon_file_system_path_for_location(fs, HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE);
+    path = _hildon_file_system_path_for_location(fs,
+             HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE, FALSE);
 
   result = gtk_file_system_path_to_filename(fs, path);
   gtk_file_path_free(path);
@@ -1870,10 +1867,24 @@ gchar *hildon_file_chooser_dialog_get_safe_folder_uri(
   fs = _hildon_file_system_model_get_file_system(self->priv->model);
   g_object_get(self->priv->model, "safe-folder", &path, NULL);
   if (path == NULL)
-    path = _hildon_file_system_path_for_location(fs, HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE);
+    path = _hildon_file_system_path_for_location(fs,
+             HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE, FALSE);
 
   result = gtk_file_system_path_to_uri(fs, path);
   gtk_file_path_free(path);
   
   return result;
 }
+
+/**
+ * hildon_file_chooser_dialog_select_text:
+ * @d: the dialog.
+ *
+ * Selects the text in input box and transfers focus there.
+ */
+void hildon_file_chooser_dialog_focus_to_input(HildonFileChooserDialog *d)
+{
+  g_return_if_fail(HILDON_IS_FILE_CHOOSER_DIALOG(d));
+  hildon_file_chooser_dialog_select_text(d->priv);
+}
+
