@@ -1,9 +1,9 @@
 /*
  * This file is part of hildon-fm package
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005-2006 Nokia Corporation.
  *
- * Contact: Luc Pionchon <luc.pionchon@nokia.com>
+ * Contact: Kimmo Hmlinen <kimmo.hamalainen@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -34,19 +34,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <gtk/gtkicontheme.h>
-#include <osso-log.h>
 #include "hildon-file-system-private.h"
 #include "hildon-file-system-settings.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "hildon-file-common-private.h"
 
 extern GtkFileSystem *gtk_file_system_unix_new();
 
-#define _(String) dgettext(PACKAGE, String)
-#define N_(String) String
-
 typedef struct
 {
   const char *icon_name;
@@ -55,7 +49,7 @@ typedef struct
 } LocationType; 
 
 /* Keep this table in sync with public enumeration found in
-    hildon-file-selection.h */
+    hildon-file-system-common.h */
 static LocationType locations[] = {
   { NULL, NULL, NULL }, 
   { "qgn_list_gene_unknown_file", NULL, NULL }, 
@@ -79,9 +73,12 @@ static const char *SOUNDS_SAFE = ".sounds";
 static const char *DOCUMENTS_SAFE = ".documents";
 static const char *GAMES_SAFE = ".games";
 
-/* Let's make sure that we survise with folder names both ending and not ending to slash */
-gboolean 
-_hildon_file_system_compare_ignore_last_separator(const char *a, const char *b)
+static const char *mmc2_path = NULL;
+
+/* Let's make sure that we survise with folder names both ending and
+ * not ending to slash */
+gboolean _hildon_file_system_compare_ignore_last_separator(const char *a,
+                                                           const char *b)
 {        
     gint len_a, len_b;
 
@@ -89,9 +86,9 @@ _hildon_file_system_compare_ignore_last_separator(const char *a, const char *b)
     len_b = strlen(b);
 
     if (len_a > 1 && a[len_a - 1] == G_DIR_SEPARATOR)
-        len_a--;
+      len_a--;
     if (len_b > 1 && b[len_b - 1] == G_DIR_SEPARATOR)
-        len_b--;
+      len_b--;
 
     if (len_a != len_b)
       return FALSE;
@@ -119,7 +116,8 @@ void _hildon_file_system_ensure_locations(void)
     if (env && env[0])
       root = g_build_path(G_DIR_SEPARATOR_S, env, LOCAL_DEVICE_ROOT, NULL);
     else
-      root = g_build_path(G_DIR_SEPARATOR_S, g_get_home_dir(), LOCAL_DEVICE_ROOT, NULL);
+      root = g_build_path(G_DIR_SEPARATOR_S, g_get_home_dir(),
+                          LOCAL_DEVICE_ROOT, NULL);
   }
 
   locations[HILDON_FILE_SYSTEM_MODEL_LOCAL_DEVICE].path = root;
@@ -139,10 +137,18 @@ void _hildon_file_system_ensure_locations(void)
     env = "/media/mmc1";    /* Fallback location */
 
   locations[HILDON_FILE_SYSTEM_MODEL_MMC].path = g_strdup(env);
+
+  env = g_getenv("INTERNAL_MMC_MOUNTPOINT");
+  if (env != NULL)
+  {
+    locations[HILDON_FILE_SYSTEM_MODEL_MMC].display_name =
+      N_("sfil_li_memorycard_internal");
+    mmc2_path = g_strdup(env);
+  }
 }
 
-gint 
-_hildon_file_system_get_special_location(GtkFileSystem *fs, const GtkFilePath *path)
+gint _hildon_file_system_get_special_location(GtkFileSystem *fs,
+                                              const GtkFilePath *path)
 {
   gchar *local_path;
   gint i, result = HILDON_FILE_SYSTEM_MODEL_UNKNOWN;
@@ -150,9 +156,16 @@ _hildon_file_system_get_special_location(GtkFileSystem *fs, const GtkFilePath *p
   local_path = gtk_file_system_path_to_filename(fs, path);
   if (local_path)
   {
+    if (mmc2_path != NULL && strcmp(local_path, mmc2_path) == 0)
+    {
+      g_free(local_path);  
+      return -1;
+    }
+
     for (i = 3; i < G_N_ELEMENTS(locations); i++)
       if (locations[i].path && 
-          _hildon_file_system_compare_ignore_last_separator(local_path, locations[i].path))
+          _hildon_file_system_compare_ignore_last_separator(local_path,
+                                                      locations[i].path))
       {
         result = i;
         break;
@@ -301,7 +314,7 @@ GdkPixbuf *_hildon_file_system_load_icon_cached(GtkIconTheme *theme,
 GdkPixbuf *
 _hildon_file_system_create_image(GtkFileSystem *fs, 
       GtkWidget *ref_widget, GtkFilePath *path, 
-      HildonFileSystemModelItemType type, gint size)
+      HildonFileSystemModelItemType type, gint size, gboolean mmc2)
 {
     GdkPixbuf *pixbuf = NULL;
 
@@ -319,7 +332,7 @@ _hildon_file_system_create_image(GtkFileSystem *fs,
         g_assert(path != NULL);
 
         vol = _hildon_file_system_get_volume_for_location(fs, 
-            HILDON_FILE_SYSTEM_MODEL_GATEWAY, path);
+            HILDON_FILE_SYSTEM_MODEL_GATEWAY, path, mmc2);
 
         if (vol)
         {
@@ -329,8 +342,17 @@ _hildon_file_system_create_image(GtkFileSystem *fs,
       }  
 
       if (!pixbuf)
-          pixbuf = _hildon_file_system_load_icon_cached(gtk_icon_theme_get_default(), 
-                        locations[type].icon_name, size);
+      {
+        if (!mmc2)
+          pixbuf = _hildon_file_system_load_icon_cached(
+                     gtk_icon_theme_get_default(), 
+                     locations[type].icon_name, size);
+        else
+          pixbuf = _hildon_file_system_load_icon_cached(
+                     gtk_icon_theme_get_default(),     
+                     locations[HILDON_FILE_SYSTEM_MODEL_MMC].icon_name, size);
+
+      }
     }
 
     return pixbuf;
@@ -360,16 +382,16 @@ static const gchar *get_custom_root_name(const GtkFilePath *path)
 gchar *
 _hildon_file_system_create_file_name(GtkFileSystem *fs,
   const GtkFilePath *path, HildonFileSystemModelItemType type,
-  GtkFileInfo *info)
+  GtkFileInfo *info, gboolean mmc2)
 {
   gchar *str;
 
   if (type == HILDON_FILE_SYSTEM_MODEL_GATEWAY ||
-      type == HILDON_FILE_SYSTEM_MODEL_MMC)
+      type == HILDON_FILE_SYSTEM_MODEL_MMC || mmc2)
   {
     GtkFileSystemVolume *vol;
 
-    vol = _hildon_file_system_get_volume_for_location(fs, type, path);
+    vol = _hildon_file_system_get_volume_for_location(fs, type, path, mmc2);
 
     if (vol)
     {
@@ -381,8 +403,15 @@ _hildon_file_system_create_file_name(GtkFileSystem *fs,
       {
         g_strstrip(str);
 
-        if (str[0] && strcmp(str, "mmc-undefined-name") != 0) 
+        if (str[0] && strcmp(str, "mmc-undefined-name") == 0) 
+        {
+          g_free(str);
+          return g_strdup(_("sfil_li_mmc_localdevice"));
+        }
+        else if (str[0])
+        {
           return str;
+        }
 
         g_free(str);
       }
@@ -403,6 +432,15 @@ _hildon_file_system_create_file_name(GtkFileSystem *fs,
     }
   }
 
+  if (mmc2)
+  {
+    const gchar *name;
+    name = _("sfil_li_memorycard_removable");
+    g_return_val_if_fail(name, NULL);
+
+    return g_strdup(name);
+  }
+
   if (type > HILDON_FILE_SYSTEM_MODEL_FOLDER)
   {
     const gchar *name;
@@ -429,12 +467,12 @@ _hildon_file_system_create_file_name(GtkFileSystem *fs,
 gchar *
 _hildon_file_system_create_display_name(GtkFileSystem *fs, 
   const GtkFilePath *path, HildonFileSystemModelItemType type, 
-  GtkFileInfo *info)
+  GtkFileInfo *info, gboolean mmc2)
 {
   gchar *str, *dot;
   const gchar *mime_type = NULL;
 
-  str = _hildon_file_system_create_file_name(fs, path, type, info);   
+  str = _hildon_file_system_create_file_name(fs, path, type, info, mmc2);   
 
   if (info)
     mime_type = gtk_file_info_get_mime_type(info);
@@ -449,17 +487,20 @@ _hildon_file_system_create_display_name(GtkFileSystem *fs,
 }
 
 GtkFilePath *_hildon_file_system_path_for_location(GtkFileSystem *fs, 
-  HildonFileSystemModelItemType type)
+  HildonFileSystemModelItemType type, gboolean mmc2)
 {
   g_assert(0 <= type && type < G_N_ELEMENTS(locations));
-  return gtk_file_system_filename_to_path(fs, locations[type].path);
+  if (!mmc2)
+    return gtk_file_system_filename_to_path(fs, locations[type].path);
+  else
+    return gtk_file_system_filename_to_path(fs, mmc2_path);
 }
 
 /* You can omit either type or base */
 GtkFileSystemVolume *
 _hildon_file_system_get_volume_for_location(GtkFileSystem *fs, 
   HildonFileSystemModelItemType type,
-  const GtkFilePath *base)
+  const GtkFilePath *base, gboolean mmc2)
 {
     GSList *volumes, *iter;
     GtkFilePath *mount_path, *path;
@@ -471,7 +512,8 @@ _hildon_file_system_get_volume_for_location(GtkFileSystem *fs,
     volumes = gtk_file_system_list_volumes(fs);
 
     mount_path = base ? gtk_file_path_copy(base) : 
-        _hildon_file_system_path_for_location(fs, type);
+                 _hildon_file_system_path_for_location(fs, type, mmc2);
+
     if (mount_path == NULL) return NULL;
 
     path_a = gtk_file_path_get_string(mount_path);
@@ -528,7 +570,7 @@ GtkFileSystem *hildon_file_system_create_backend(const gchar *name, gboolean use
         name = default_name;
     }
     if (name) {
-        result = _gtk_file_system_create(name);
+        result = hildon_gtk_file_system_create(name);
         if (!GTK_IS_FILE_SYSTEM(result))
             ULOG_WARN("Cannot create \"%s\" backend", name);
     }
